package com.htfp.service.usm.app.handler;

import com.htfp.service.usm.app.model.BaseHttpResponse;
import com.htfp.service.usm.biz.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.StringUtils;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.validation.ConstraintViolationException;
import java.sql.SQLException;
import java.util.stream.Collectors;

import com.htfp.service.usm.common.enums.ErrorCodeEnum;
import org.springframework.web.method.HandlerMethod;

/**
 * @author : shi_y
 * @description : 全局异常处理器
 * @createTime : 2025-06-11 11:33
 **/
@RestControllerAdvice
@Slf4j
public class GlobalExceptionHandler {

    // 参数校验失败：@Valid/@Validated + @RequestBody
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public BaseHttpResponse<Void> handleMethodArgumentNotValidException(MethodArgumentNotValidException ex, HandlerMethod handlerMethod) {
        StringBuilder errMsgBuilder = new StringBuilder();
        for (FieldError error : ex.getBindingResult().getFieldErrors()) {
            errMsgBuilder
                    // .append(error.getField())
                    // .append(": ")
                    .append(error.getDefaultMessage()).append("; ");
        }
        // 获取 controller 类名 + 方法名
        String beanName = handlerMethod.getBeanType().getName();
        String methodName = handlerMethod.getMethod().getName();
        String errMsg = errMsgBuilder.toString();
        log.warn("参数校验失败:\n    Path: {} - {},\n    错误信息: {}", beanName, methodName, errMsg);
        return BaseHttpResponse.fail(ErrorCodeEnum.PARAM_VALIDATION_ERROR, errMsg);
    }

    // 参数校验失败：@Validated + @RequestParam / @PathVariable
    @ExceptionHandler(ConstraintViolationException.class)
    public BaseHttpResponse<Void> handleConstraintViolationException(ConstraintViolationException ex, HandlerMethod handlerMethod) {
        String message = ex.getConstraintViolations()
                .stream()
                .map(v -> v.getPropertyPath() + ": " + v.getMessage())
                .collect(Collectors.joining("; "));
        String beanName = handlerMethod.getBeanType().getName();
        String methodName = handlerMethod.getMethod().getName();
        log.warn("路径参数校验失败:\n    Path: {} - {},\n    错误信息: {}", beanName, methodName, message);
        return BaseHttpResponse.fail(ErrorCodeEnum.PARAM_VALIDATION_ERROR, message);
    }

    @ExceptionHandler(BusinessException.class)
    public BaseHttpResponse<Void> handleBusinessException(BusinessException ex, HandlerMethod handlerMethod) {
        ErrorCodeEnum codeEnum = ex.getErrorCodeEnum();
        String content = ex.getcontent();
        String finalMessage;

        String beanName = handlerMethod.getBeanType().getName();
        String methodName = handlerMethod.getMethod().getName();

        BaseHttpResponse<Void> response;
        if (!StringUtils.hasLength(content)) {
            response =  BaseHttpResponse.fail(codeEnum);
        } else {
            finalMessage = String.format("%s: %s", codeEnum.getDesc(), content);
            response =  BaseHttpResponse.fail(codeEnum, finalMessage);
        }
        log.warn("业务异常:\n    Path: {} - {},\n    错误信息: {}", beanName, methodName, response);
        return response;
    }

    @ExceptionHandler(SQLException.class)
    public BaseHttpResponse<Void> handleSQLException(SQLException ex, HandlerMethod handlerMethod) {
        String beanName = handlerMethod.getBeanType().getName();
        String methodName = handlerMethod.getMethod().getName();

        log.error("数据库异常:\n    Path: {} - {},\n    错误信息: {}", beanName, methodName, ex.getMessage());
        return BaseHttpResponse.fail(ErrorCodeEnum.SQL_ERROR, "服务器内部错误，请联系管理员");
    }
    // 系统异常兜底
    @ExceptionHandler(Exception.class)
    public BaseHttpResponse<Void> handleSystemException(Exception ex, HttpServletRequest request) {
        log.error("系统异常, path: {}, error: {}", request.getRequestURI(), ex.getMessage(), ex);
        return BaseHttpResponse.fail(ErrorCodeEnum.SYSTEM_ERROR, ex.getMessage());
    }
}
