package com.htfp.service.usm.common.utils;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.*;
import javax.validation.constraints.NotNull;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * JSON 工具类，封装 Jackson 对象序列化/反序列化，支持类型泛化。
 */
public class JsonUtils {

    private static final ObjectMapper objectMapper = new ObjectMapper();

    // 默认配置
    static {
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false); // 是否忽略未知字段
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL); //  序列化时忽略null字段
        objectMapper.enable(SerializationFeature.INDENT_OUTPUT); // 输出带格式的JSON
        objectMapper.enable(SerializationFeature.FAIL_ON_SELF_REFERENCES);
        objectMapper.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
        // objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));
    }

    private JsonUtils() {}

    /**
     * 获取 ObjectMapper单例
     */
    public static ObjectMapper getInstance() {
        return objectMapper;
    }


    /**
     * 将对象序列化为 JSON 字符串。
     *
     * @param obj 要序列化的对象
     * @return JSON 字符串；若对象为 null 返回 null
     * @throws JsonSerializeException 序列化失败
     */
    public static String obj2json(@NotNull Object obj) {
        if (obj == null) return null;
        if (obj instanceof Map && mapSelfContain((Map) obj)){
            // 循环检测
            throw new JsonSerializeException("对象序列化失败，对象中包含自身引用");
        }
        try {
            return objectMapper.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            throw new JsonSerializeException("对象序列化为 JSON 字符串失败", e);
        }
    }

    private static boolean mapSelfContain(Map obj) {
        return obj.containsValue(obj);
    }

    /**
     * 将 JSON 字符串反序列化为对象。
     *
     * @param jsonStr JSON 字符串
     * @param clazz   类型
     * @param <T>     泛型
     * @return 反序列化对象；若 jsonStr 为空返回 null
     * @throws IllegalArgumentException 参数为空
     * @throws JsonDeserializeException 解析失败
     */
    public static <T> T json2obj(@NotNull String jsonStr, @NotNull Class<T> clazz) {
        if (clazz == null) throw new IllegalArgumentException("参数 clazz 不能为空");
        if (jsonStr == null || jsonStr.trim().isEmpty()) return null;
        try {
            return objectMapper.readValue(jsonStr, clazz);
        } catch (IOException e) {
            throw new JsonDeserializeException("JSON 字符串转对象失败: " + clazz.getSimpleName(), e);
        }
    }

    /**
     * 将 JSON 字符串反序列化为泛型对象。
     *
     * @param jsonStr       JSON 字符串
     * @param typeReference 泛型引用
     * @param <T>           泛型
     * @return 泛型对象；若 jsonStr 为空返回 null
     * @throws IllegalArgumentException 参数为空
     * @throws JsonDeserializeException 解析失败
     */
    public static <T> T json2obj(@NotNull String jsonStr, @NotNull TypeReference<T> typeReference) {
        if (typeReference == null) throw new IllegalArgumentException("参数 typeReference 不能为空");
        if (jsonStr == null || jsonStr.trim().isEmpty()) return null;
        try {
            return objectMapper.readValue(jsonStr, typeReference);
        } catch (IOException e) {
            throw new JsonDeserializeException("JSON 字符串转泛型对象失败", e);
        }
    }

    /**
     * 将 JSON 字符串转换为 Map<String, Object>。
     *
     * @param jsonStr JSON 字符串
     * @return Map；若 jsonStr 为空返回 null
     * @throws JsonDeserializeException 解析失败
     */
    public static Map<String, Object> json2map(@NotNull String jsonStr) {
        return json2obj(jsonStr, new TypeReference<Map<String, Object>>() {});
    }

    /**
     * 将 JSON 字符串转换为 Map<String, T>。
     *
     * @param jsonStr JSON 字符串
     * @param clazz   值类型
     * @param <T>     泛型
     * @return Map；若 jsonStr 为空返回 null
     * @throws IllegalArgumentException 参数为空
     * @throws JsonDeserializeException 解析失败
     */
    public static <T> Map<String, T> json2map(@NotNull String jsonStr, @NotNull Class<T> clazz) {
        if (clazz == null) throw new IllegalArgumentException("参数 clazz 不能为空");
        if (jsonStr == null || jsonStr.trim().isEmpty()) return null;
        try {
            JavaType type = objectMapper.getTypeFactory().constructMapType(Map.class, String.class, clazz);
            return objectMapper.readValue(jsonStr, type);
        } catch (IOException e) {
            throw new JsonDeserializeException("JSON 字符串转 Map<String, " + clazz.getSimpleName() + "> 失败", e);
        }
    }

    /**
     * 将 JSON 数组字符串转换为 List<T>。
     *
     * @param jsonArrayStr JSON 数组字符串
     * @param clazz        元素类型
     * @param <T>          泛型
     * @return List；若为空返回 null
     * @throws IllegalArgumentException 参数为空
     * @throws JsonDeserializeException 解析失败
     */
    public static <T> List<T> json2list(@NotNull String jsonArrayStr, @NotNull Class<T> clazz) {
        if (clazz == null) throw new IllegalArgumentException("参数 clazz 不能为空");
        if (jsonArrayStr == null || jsonArrayStr.trim().isEmpty()) return null;
        try {
            JavaType type = objectMapper.getTypeFactory().constructCollectionType(List.class, clazz);
            return objectMapper.readValue(jsonArrayStr, type);
        } catch (IOException e) {
            throw new JsonDeserializeException("JSON 数组转 List<" + clazz.getSimpleName() + "> 失败", e);
        }
    }

    /**
     * 将 Map 转换为对象。
     *
     * @param map   原始 Map
     * @param clazz 目标类型
     * @param <T>   泛型
     * @return 实例；若 map 为 null 返回 null
     * @throws IllegalArgumentException 参数为空
     */
    public static <T> T map2obj(@NotNull Map<String, ?> map, @NotNull Class<T> clazz) {
        if (map == null) return null;
        if (clazz == null) throw new IllegalArgumentException("参数 clazz 不能为空");
        return objectMapper.convertValue(map, clazz);
    }

    /**
     * 将对象输出为格式化 JSON 写入文件。
     *
     * @param obj      要写入的对象
     * @param filePath 目标文件路径
     * @throws IllegalArgumentException 参数为空
     * @throws JsonSerializeException 写入失败
     */
    public static void obj2jsonFile(@NotNull Object obj, @NotNull String filePath) {
        if (obj == null) throw new IllegalArgumentException("obj 不能为空");
        if (filePath == null || filePath.trim().isEmpty()) {
            throw new IllegalArgumentException("filePath 不能为空");
        }
        try {
            objectMapper.writeValue(new File(filePath), obj);
        } catch (IOException e) {
            throw new JsonSerializeException("对象写入 JSON 文件失败: " + filePath, e);
        }
    }

    /**
     * JSON 序列化异常（运行时）
     */
    public static class JsonSerializeException extends RuntimeException {
        public JsonSerializeException(String msg, Throwable cause) {
            super(msg, cause);
        }
        public JsonSerializeException(String msg) {
            super(msg);
        }
    }

    /**
     * JSON 反序列化异常（运行时）
     */
    public static class JsonDeserializeException extends RuntimeException {
        public JsonDeserializeException(String msg, Throwable cause) {
            super(msg, cause);
        }
        public JsonDeserializeException(String msg) {
            super(msg);
        }
    }
}
