package com.htfp.service.usm.common.utils;

import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.core.type.TypeReference;
import com.htfp.service.usm.common.utils.JsonUtils.JsonDeserializeException;
import com.htfp.service.usm.common.utils.JsonUtils.JsonSerializeException;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.*;

import static org.junit.jupiter.api.Assertions.*;

class JsonUtilsTest {

    @Data @NoArgsConstructor@AllArgsConstructor
    @JsonTypeInfo(use = JsonTypeInfo.Id.CLASS)
    private static class TestBean {
        private String name;
        private int age;
        // 省略 getter/setter 以保持示例简洁


        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            TestBean testBean = (TestBean) o;

            if (age != testBean.age) return false;
            return Objects.equals(name, testBean.name);
        }

        @Override
        public int hashCode() {
            int result = name != null ? name.hashCode() : 0;
            result = 31 * result + age;
            return result;
        }
    }

    @Data
    private static class TestBeanWithList {
        private String name;
        private int age;
        private List<String> hobbies;
        // 省略 getter/setter 以保持示例简洁
    }

    // 测试泛型结构
    @Data
    private static class TestBeanWithGeneric <K>  {
        private K key;
        // 省略 getter/setter 以保持示例简洁
    }

    // 测试循环引用
    @Data
    private static class CircularReference {
        private CircularReference self;
        // 省略 getter/setter 以保持示例简洁
    }

    @Data @NoArgsConstructor @AllArgsConstructor
    private static class TestChildBeanA extends TestBean {
        private String fieldChildA;
    }
    @Data @NoArgsConstructor @AllArgsConstructor
    private static class TestChildBeanB extends TestBean {
        private String fieldChildB;
    }
    // 测试 obj2json 正常序列化
    @Test
    void testObj2json_Normal() {
        TestBean bean = new TestBean("Alice", 30);
        String json = JsonUtils.obj2json(bean);
        assertNotNull(json);
        assertTrue(json.contains("name") && json.contains("age"));
    }

    // 测试 obj2json 对 null 的处理
    @Test
    void testObj2json_Null() {
        assertNull(JsonUtils.obj2json(null));
    }

    // 测试 obj2json 抛出异常（当输入对象包含循环引用时）
    @Test
    void testObj2json_Exception() {
        // 创建一个带有循环引用的对象
        Map<String, Object> map = new HashMap<>();
        map.put("map", map); // 循环引用
        
        assertThrows(JsonSerializeException.class, () -> JsonUtils.obj2json(map));
    }

    // 测试继承关系序列化
    @Test
    void testObj2json_ParentChild() {
        TestChildBeanA childA = new TestChildBeanA();
        childA.setName("childA");
        childA.setAge(40);
        childA.setFieldChildA("fieldChildA");

        TestChildBeanB childB = new TestChildBeanB();
        childB.setName("childB");
        childB.setAge(30);
        childB.setFieldChildB("fieldChildB");

        List<TestBean> beanList  = Arrays.asList(childA, childB);
        String json = JsonUtils.obj2json(beanList);
        System.out.println(json);
        assertNotNull(json);
        assertTrue(json.contains("childA") && json.contains("childB"));
        assertTrue(json.contains("fieldChildA") && json.contains("fieldChildB"));
    }

    // 测试继承关系反序列化
    // @Test
    // void testJson2obj_ParentChild() {
    //     String json = "[{\"name\":\"childA\",\"age\":40,\"fieldChildA\":\"fieldChildA\"},{\"name\":\"childB\",\"age\":30,\"fieldChildB\":\"fieldChildB\"}]";
    //
    //     List<TestBean> beanList = JsonUtils.json2list(json, TestBean.class);
    //     assertNotNull(beanList);
    //     assertEquals(2, beanList.size());
    //     assertEquals("childA", beanList.get(0).getName());
    //     assertEquals(40, beanList.get(0).getAge());
    //     assertEquals("fieldChildA", ((TestChildBeanA) beanList.get(0)).getFieldChildA());
    //     assertEquals("childB", beanList.get(1).getName());
    // }


    @Test
    void testObj2json_SelfReference() {
        // 创建一个带有循环引用的对象
        CircularReference circularReference = new CircularReference();
        circularReference.self = circularReference;
        assertThrows(JsonSerializeException.class, () -> JsonUtils.obj2json(circularReference));
    }

    
    // 测试 json2obj 正常反序列化
    @Test
    void testJson2obj_StringClass_Normal() {
        String json = "{\"name\":\"Bob\",\"age\":25}";
        TestBean bean = JsonUtils.json2obj(json, TestBean.class);
        assertNotNull(bean);
        assertEquals("Bob", bean.name);
        assertEquals(25, bean.age);
    }

    // 测试 json2obj 对空字符串的处理
    @Test
    void testJson2obj_StringClass_EmptyString() {
        assertNull(JsonUtils.json2obj("", TestBean.class));
    }

    // 测试 json2obj 对 null 的处理
    @Test
    void testJson2obj_StringClass_Null() {
        assertNull(JsonUtils.json2obj(null, TestBean.class));
    }

    // 测试 json2obj 泛型版本正常反序列化
    @Test
    void testJson2obj_TypeReference_Normal() {
        String json = "{\"name\":\"Charlie\",\"age\":40}";
        TestBean bean = JsonUtils.json2obj(json, new TypeReference<TestBean>() {});
        assertNotNull(bean);
        assertEquals("Charlie", bean.name);
        assertEquals(40, bean.age);
    }
    
    // 测试 json2obj 泛型版本对嵌套结构的处理
    @Test
    void testJson2obj_TypeReference_Nested() { 
        String jsonString = "{\"key\":\"ok\"}";
        TestBeanWithGeneric<String> bean = JsonUtils.json2obj(jsonString, new TypeReference<TestBeanWithGeneric<String>>() {});
        assertNotNull(bean);
        assertEquals("ok", bean.key);
        
        String jsonInteger = "{\"key\":10}";
        TestBeanWithGeneric<Integer> beanWithInt = JsonUtils.json2obj(jsonInteger, new TypeReference<TestBeanWithGeneric<Integer>>() {});
        assertNotNull(beanWithInt);
    }

    // 测试 json2map 反序列化
    @Test
    void testJson2map() {
        String json = "{\"key1\":\"value1\",\"key2\":\"value2\"}";
        Map<String, Object> map = JsonUtils.json2map(json);
        assertNotNull(map);
        assertEquals(2, map.size());
        assertEquals("value1", map.get("key1"));
        assertEquals("value2", map.get("key2"));
    }

    // 测试 json2map 带类型参数
    @Test
    void testJson2map_Class() {
        String json = "{\"key1\":20,\"key2\":30}";
        Map<String, Integer> map = JsonUtils.json2map(json, Integer.class);
        assertNotNull(map);
        assertEquals(2, map.size());
        assertEquals(Integer.valueOf(20), map.get("key1"));
        assertEquals(Integer.valueOf(30), map.get("key2"));
    }

    // 测试 json2list 反序列化
    @Test
    void testJson2list() {
        String json = "[\"item1\",\"item2\",\"item3\"]";
        List<String> list = JsonUtils.json2list(json, String.class);
        assertNotNull(list);
        assertEquals(3, list.size());
        assertEquals("item1", list.get(0));
        assertEquals("item2", list.get(1));
        assertEquals("item3", list.get(2));
    }

    // 测试objList反序列化
    @Test
    void testJson2objList() {
        // List<TestBean>
        String json = "[{\"name\":\"Alice\",\"age\":30},{\"name\":\"Bob\",\"age\":25}]";
        List<TestBean> list = JsonUtils.json2list(json, TestBean.class);
        assertNotNull(list);
        assertEquals(2, list.size());
        assertEquals("Alice", list.get(0).name);
        assertEquals(30, list.get(0).age);
        assertEquals("Bob", list.get(1).name);
        assertEquals(25, list.get(1).age);
    }
    @Test
    void testJson2objListUseTypeReference() {
        // List<TestBean>
        String json = "[{\"name\":\"Alice\",\"age\":30},{\"name\":\"Bob\",\"age\":25}]";
        List<TestBean> list = JsonUtils.json2obj(json, new TypeReference<List<TestBean>>() {});
        assertNotNull(list);
        assertEquals(2, list.size());
        assertEquals("Alice", list.get(0).name);
        assertEquals(30, list.get(0).age);
        assertEquals("Bob", list.get(1).name);
        assertEquals(25, list.get(1).age);
    }

    // 测试 map2obj 转换
    @Test
    void testMap2obj() {
        Map<String, Object> map = new HashMap<>();
        map.put("name", "David");
        map.put("age", 35);
        
        TestBean bean = JsonUtils.map2obj(map, TestBean.class);
        assertNotNull(bean);
        assertEquals("David", bean.name);
        assertEquals(35, bean.age);
    }

    // 测试 map2obj 对 null 的处理
    @Test
    void testMap2obj_NullMap() {
        assertNull(JsonUtils.map2obj(null, TestBean.class));
    }

    // 测试 map2obj 对 null 类的处理
    @Test
    void testMap2obj_NullClass() {
        assertThrows(IllegalArgumentException.class, () -> {
            JsonUtils.map2obj(new HashMap<>(), null);
        });
    }
}